==> idealyzer-1.0.6.tar.gz: idealyzer/src/Controller/ResultController.php <==
<?php

namespace Drupal\idealyzer\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\State\StateInterface;
use Drupal\Core\File\FileSystemInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class ResultController extends ControllerBase {
    protected $state;
    protected $fileSystem;

    public function __construct(StateInterface $state, FileSystemInterface $fileSystem) {
        $this->state = $state;
        $this->fileSystem = $fileSystem;
    }

    public static function create(ContainerInterface $container) {
        return new static(
            $container->get('state'),
            $container->get('file_system')
        );
    }

    public function displayResults() {
        $data = $this->state->get('idealyzer.api_response', []);

        // Log the retrieved data
        \Drupal::logger('idealyzer')->debug('Retrieved data: <pre>' . print_r($data, TRUE) . '</pre>');

        $build = [
            'clear_results' => [
                '#type' => 'link',
                '#title' => $this->t('Clear Results'),
                '#url' => \Drupal\Core\Url::fromRoute('idealyzer.clear_results'),
                '#attributes' => ['class' => ['button']],
            ],
            'download_link' => $this->buildDownloadLink(),
        ];

        if (!empty($data['items'])) {
            // Extract scan info
            $build['scan_information'] = $this->buildScanInformation($data['items'][0]);

          

            // Build categorized tables
            $categorizedItems = $this->parseItems($data['items']);
            foreach ($categorizedItems as $category => $items) {
                $build[$category] = [
                    '#type' => 'details',
                    '#title' => $this->t($category),
                    '#open' => true,
                    'table' => [
                        '#type' => 'table',
                        '#header' => $this->getTableHeaders(),
                        '#rows' => $this->formatRowsForTable($items),
                        '#empty' => $this->t('No results for this category.'),
                    ],
                ];
            }
        } else {
            $build = [
                '#type' => 'markup',
                '#markup' => $this->t('No items found.'),
                'scan' => [
                    '#type' => 'link',
                    '#title' => $this->t('Return to Scanner'),
                    '#url' => \Drupal\Core\Url::fromRoute('idealyzer.admin_form'),
                    '#attributes' => ['class' => ['button', 'button--primary']],
                ],
            ];
        }

        return $build;
    }

    
    private function buildScanInformation($item) {
        $scanTitle = $this->formatValue($item['title'] ?? 'N/A');
        $scanDate = $this->formatValue($item['scan_date'] ?? 'N/A');
        $targetUrlDomain = $this->formatValue($item['target_url_domain'] ?? 'N/A');
        $completedScans = 0;
        $totalScans = 0;

        $scanStatuses = [
            'not_found_scan_status',
            'primary_scan_status',
            'robots_txt_scan_status',
            'sitemap_xml_scan_status',
            'dns_scan_status',
            'accessibility_scan_status',
            'performance_scan_status',
            'security_scan_status',
        ];

        foreach ($scanStatuses as $status) {
            if (isset($item[$status])) {
                $totalScans++;
                if ($item[$status] === 'completed') {
                    $completedScans++;
                }
            }
        }

        $overviewText = [
            $this->t('Scan Overview: @domain - @title', ['@domain' => $targetUrlDomain, '@title' => $scanTitle]),
            $this->t('Scan Date: @date', ['@date' => $scanDate]),
            $this->t('Completed Scans: @completed/@total', [
                '@completed' => $completedScans,
                '@total' => $totalScans,
            ]),
        ];

        return [
            '#type' => 'table',
            '#header' => [$overviewText[0], ''],
            '#rows' => [
                [
                    [
                        'data' => $overviewText[1],
                        'class' => 'scan-info-key',
                    ],
                    [
                        'data' => $overviewText[2],
                        'class' => 'scan-info-value',
                    ],
                ],
            ],
        ];
    }

    private function parseItems($items) {
        $categories = [
            'Domain Information' => ['dns_scan_status', 'target_url_domain', 'final_url', 'final_url_status_code', 'ipv6', 'final_url_media_type', 'final_url_top_level_domain', 'target_url', 'target_url_top_level_domain', 'target_url_branch', 'target_url_agency_owner', 'target_url_bureau_owner', 'cms', 'source_list', 'public'],
            'Security' => ['security_scan_status', 'https_enforced', 'hsts', 'robots_txt_scan_status', 'robots_txt_final_url', 'robots_txt_final_url_status_code', 'robots_txt_final_url_live', 'robots_txt_detected', 'robots_txt_final_url_filesize', 'sitemap_xml_scan_status', 'sitemap_xml_final_url', 'sitemap_xml_final_url_status_code', 'sitemap_xml_final_url_live', 'sitemap_xml_detected', 'sitemap_xml_final_url_filesize', 'sitemap_xml_count', 'sitemap_xml_pdf_count'],
            'Content' => ['dap', 'dap_parameters', 'og_title', 'og_description', 'main_element_present', 'title', 'description', 'language', 'viewport_meta_tag', 'canonical_link', 'required_links_url', 'required_links_text'],
            'Performance' => ['performance_scan_status', 'largest_contentful_paint', 'cumulative_layout_shift', 'final_url_live', 'final_url_same_domain', 'final_url_same_website', 'target_url_404_test', 'target_url_redirects', 'hostname'],
            'USWDS' => ['uswds_usa_classes', 'uswds_string', 'uswds_inpage_css', 'uswds_favicon', 'uswds_string_in_css', 'uswds_favicon_in_css', 'uswds_publicsans_font', 'uswds_semantic_version', 'uswds_version', 'uswds_count']
        ];

        $categorizedItems = [];
        foreach ($categories as $category => $keys) {
            $categorizedItems[$category] = [];
        }

        $item = $items[0] ?? [];

        foreach ($item as $key => $value) {
            foreach ($categories as $category => $keys) {
                if (in_array($key, $keys)) {
                    $categorizedItems[$category][] = [$key => $this->formatValue($value)];
                }
            }
        }

        return $categorizedItems;
    }

    private function formatValue($value) {
        if ($value === null) {
            return 'N/A';
        }
        if (is_array($value)) {
            return implode(', ', array_map([$this, 'formatValue'], $value));
        }
        return htmlspecialchars((string) $value, ENT_QUOTES, 'UTF-8');
    }

    private function getTableHeaders() {
        return ['Field', 'Value'];
    }

    private function determineStyle($value) {
        switch ($value) {
            case 'completed':
            case 'true':
                return ['icon' => 'check-circle', 'class' => 'pass'];
            case 'failed':
            case 'false':
                return ['icon' => 'times-circle', 'class' => 'fail'];
            case 'N/A':
            case null:
                return ['icon' => 'question-circle', 'class' => 'na'];
            default:
                return ['icon' => 'info-circle', 'class' => 'info'];
        }
    }

    private function formatRowsForTable($items) {
        $rows = [];
        foreach ($items as $item) {
            foreach ($item as $key => $value) {
                $styleInfo = $this->determineStyle($value);
                $rows[] = [
                    [
                        'data' => [
                            '#type' => 'html_tag',
                            '#tag' => 'span',
                            '#value' => $key,
                            '#attributes' => ['class' => [$styleInfo['class']]]
                        ],
                    ],
                    [
                        'data' => [
                            '#markup' => $this->t('<i class="fa fa-:icon"></i> :value', [
                                ':icon' => $styleInfo['icon'], 
                                ':value' => $value
                            ]),
                            '#allowed_tags' => ['i'],
                        ],
                    ],
                ];
            }
        }
        return $rows;
    }

    private function buildDownloadLink() {
        $filePaths = $this->state->get('idealyzer.last_scan_files', []);
        if (!empty($filePaths)) {
            return [
                '#type' => 'link',
                '#title' => $this->t('Download Results'),
                '#url' => \Drupal\Core\Url::fromRoute('idealyzer.download_results'),
                '#attributes' => ['class' => ['button', 'button--primary']],
            ];
        }
        return [];
    }

    public function processResults($data) {
        // Clear previous results and files
        $this->clearResults();

        // Save new results and update the state with file paths
        $filePaths = $this->saveResults($data);
        $this->state->set('idealyzer.api_response', $data);
        $this->state->set('idealyzer.last_scan_files', $filePaths);

        $this->messenger()->addMessage($this->t('Scan results saved. Download links are now available.'));
    }

    public function clearResults() {
        // Clear the stored results state
        $this->state->delete('idealyzer.api_response');
        $this->state->delete('idealyzer.last_scan_files');

        // Optionally delete files linked to old results
        $this->deleteOldFiles();
        return $this->redirect('idealyzer.admin_form');
    }

    private function deleteOldFiles() {
        $previousFiles = $this->state->get('idealyzer.previous_scan_files', []);
        foreach ($previousFiles as $file) {
            $this->fileSystem->delete($file);
        }
        $this->state->delete('idealyzer.previous_scan_files');
    }
    private function saveResults($data) {
        $timestamp = date('Ymd_His');
        $json_filename = "scan_results_{$timestamp}.json";
        $csv_filename = "scan_results_{$timestamp}.csv";
    
        // Directory check
        $directory = "public://scan_results/";
        \Drupal::service('file_system')->prepareDirectory($directory, \Drupal\Core\File\FileSystemInterface::CREATE_DIRECTORY);
    
        // Save JSON format
        $json_filepath = $directory . $json_filename;
        file_put_contents($json_filepath, json_encode($data));
    
        // Save CSV format
        $csv_filepath = $directory . $csv_filename;
        $fp = fopen($csv_filepath, 'w');
    
        if (!empty($data['items'])) {
            $headers = array_keys($data['items'][0]);
            fputcsv($fp, $headers); // Write headers
            foreach ($data['items'] as $row) {
                // Ensure each row is an array
                $row = array_map(function ($value) {
                    return is_array($value) ? json_encode($value) : $value;
                }, $row);
                fputcsv($fp, $row); // Write each row
            }
        }
    
        fclose($fp);
    
        return [
            'json' => $json_filepath,
            'csv' => $csv_filepath
        ];
    }
    public function downloadResults() {
        $filePaths = $this->state->get('idealyzer.last_scan_files', []);


        if (!empty($filePaths)) {
            $links = [];
            foreach ($filePaths as $format => $path) {
                $url = \Drupal::service('file_url_generator')->generateAbsoluteString($path);
                $links[] = [
                    '#title' => $this->t('Download :format', [':format' => strtoupper($format)]),
                    '#type' => 'link',
                    '#url' => \Drupal\Core\Url::fromUri($url),
                ];
            }
            $build['downloads'] = [
                '#theme' => 'item_list',
                '#items' => $links,
                '#attributes' => ['class' => ['download-links']],
            ];
        }
        else{
            $build = [
                'scan' => [
                    '#type' => 'link',
                    '#title' => $this->t('Return to Site Scanner'),
                    '#url' => \Drupal\Core\Url::fromRoute('idealyzer.admin_form'),
                    '#attributes' => ['class' => ['button', 'button--primary']],
                ],
                '#type' => 'markup',
                '#markup' => $this->t('No items found. '),
            ];
        }

        return $build;
    }
}

==> idealyzer-1.0.9.tar.gz: idealyzer/src/Controller/ResultController.php <==
<?php

namespace Drupal\idealyzer\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\State\StateInterface;
use Drupal\Core\File\FileSystemInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\idealyzer\Service\SiteScannerService;

class ResultController extends ControllerBase {
    protected $state;
    protected $fileSystem;
    protected $scannerService;

    public function __construct(StateInterface $state, FileSystemInterface $fileSystem, SiteScannerService $scannerService) {
        $this->state = $state;
        $this->fileSystem = $fileSystem;
        $this->scannerService = $scannerService;
    }

    public static function create(ContainerInterface $container) {
        return new static(
            $container->get('state'),
            $container->get('file_system'),
            $container->get('idealyzer.site_scanner')
        );
    }

    public function displayResults() {
        $data = $this->state->get('idealyzer.api_response', []);

        // Log the retrieved data
        \Drupal::logger('idealyzer')->debug('Retrieved data: <pre>' . print_r($data, TRUE) . '</pre>');

        $build = [
            'clear_results' => [
                '#type' => 'link',
                '#title' => $this->t('Clear Results'),
                '#url' => \Drupal\Core\Url::fromRoute('idealyzer.clear_results'),
                '#attributes' => ['class' => ['button']],
            ],
            'download_link' => $this->buildDownloadLink(),
        ];

        if (!empty($data['items'])) {
            // Extract scan info
            $build['scan_information'] = $this->buildScanInformation($data['items'][0]);

            // Build categorized tables
            $categorizedItems = $this->parseItems($data['items']);
            foreach ($categorizedItems as $category => $items) {
                $build[$category] = [
                    '#type' => 'details',
                    '#title' => $this->t($category),
                    '#open' => true,
                    'table' => [
                        '#type' => 'table',
                        '#header' => $this->getTableHeaders(),
                        '#rows' => $this->formatRowsForTable($items),
                        '#empty' => $this->t('No results for this category.'),
                    ],
                ];
            }
        } else {
            $build = [
                '#type' => 'markup',
                '#markup' => $this->t('No items found.'),
                'scan' => [
                    '#type' => 'link',
                    '#title' => $this->t('Return to Scanner'),
                    '#url' => \Drupal\Core\Url::fromRoute('idealyzer.admin_form'),
                    '#attributes' => ['class' => ['button', 'button--primary']],
                ],
            ];
        }

        return $build;
    }

    public function processResults($targetSubdomain) {
        $page = 1;
        $limit = 1;
        $found = false;

        while (!$found) {
            $result = $this->scannerService->fetchData($targetSubdomain, $page, $limit);

            if (!$result['success']) {
                $this->messenger()->addError($result['error']);
                return;
            }

            if (empty($result['data']['items'])) {
                $this->messenger()->addWarning($this->t('No more results. Subdomain not found.'));
                return;
            }

            foreach ($result['data']['items'] as $item) {
                if (strpos($item['target_url'], $targetSubdomain) !== false) {
                    $this->state->set('idealyzer.api_response', $result['data']);
                    $this->messenger()->addStatus($this->t('Subdomain found and results processed.'));
                    return;
                }
            }

            $page++;
        }

        $this->messenger()->addWarning($this->t('Subdomain not found.'));
    }

    private function buildScanInformation($item) {
        $scanTitle = $this->formatValue($item['title'] ?? 'N/A');
        $scanDate = $this->formatValue($item['scan_date'] ?? 'N/A');
        $targetUrlDomain = $this->formatValue($item['target_url_domain'] ?? 'N/A');
        $completedScans = 0;
        $totalScans = 0;

        $scanStatuses = [
            'not_found_scan_status',
            'primary_scan_status',
            'robots_txt_scan_status',
            'sitemap_xml_scan_status',
            'dns_scan_status',
            'accessibility_scan_status',
            'performance_scan_status',
            'security_scan_status',
        ];

        foreach ($scanStatuses as $status) {
            if (isset($item[$status])) {
                $totalScans++;
                if ($item[$status] === 'completed') {
                    $completedScans++;
                }
            }
        }

        $overviewText = [
            $this->t('Scan Overview: @domain - @title', ['@domain' => $targetUrlDomain, '@title' => $scanTitle]),
            $this->t('Scan Date: @date', ['@date' => $scanDate]),
            $this->t('Completed Scans: @completed/@total', [
                '@completed' => $completedScans,
                '@total' => $totalScans,
            ]),
        ];

        return [
            '#type' => 'table',
            '#header' => [$overviewText[0], ''],
            '#rows' => [
                [
                    [
                        'data' => $overviewText[1],
                        'class' => 'scan-info-key',
                    ],
                    [
                        'data' => $overviewText[2],
                        'class' => 'scan-info-value',
                    ],
                ],
            ],
        ];
    }

    private function parseItems($items) {
        $categories = [
            'Domain Information' => ['dns_scan_status', 'target_url_domain', 'final_url', 'final_url_status_code', 'ipv6', 'final_url_media_type', 'final_url_top_level_domain', 'target_url', 'target_url_top_level_domain', 'target_url_branch', 'target_url_agency_owner', 'target_url_bureau_owner', 'cms', 'source_list', 'public'],
            'Security' => ['security_scan_status', 'https_enforced', 'hsts', 'robots_txt_scan_status', 'robots_txt_final_url', 'robots_txt_final_url_status_code', 'robots_txt_final_url_live', 'robots_txt_detected', 'robots_txt_final_url_filesize', 'sitemap_xml_scan_status', 'sitemap_xml_final_url', 'sitemap_xml_final_url_status_code', 'sitemap_xml_final_url_live', 'sitemap_xml_detected', 'sitemap_xml_final_url_filesize', 'sitemap_xml_count', 'sitemap_xml_pdf_count'],
            'Content' => ['dap', 'dap_parameters', 'og_title', 'og_description', 'main_element_present', 'title', 'description', 'language', 'viewport_meta_tag', 'canonical_link', 'required_links_url', 'required_links_text'],
            'Performance' => ['performance_scan_status', 'largest_contentful_paint', 'cumulative_layout_shift', 'final_url_live', 'final_url_same_domain', 'final_url_same_website', 'target_url_404_test', 'target_url_redirects', 'hostname'],
            'USWDS' => ['uswds_usa_classes', 'uswds_string', 'uswds_inpage_css', 'uswds_favicon', 'uswds_string_in_css', 'uswds_favicon_in_css', 'uswds_publicsans_font', 'uswds_semantic_version', 'uswds_version', 'uswds_count']
        ];

        $categorizedItems = [];
        foreach ($categories as $category => $keys) {
            $categorizedItems[$category] = [];
        }

        foreach ($items as $item) {
            foreach ($item as $key => $value) {
                foreach ($categories as $category => $keys) {
                    if (in_array($key, $keys)) {
                        $categorizedItems[$category][] = [$key => $this->formatValue($value)];
                    }
                }
            }
        }

        return $categorizedItems;
    }

    private function formatValue($value) {
        if ($value === null) {
            return 'N/A';
        }
        if (is_array($value)) {
            return implode(', ', array_map([$this, 'formatValue'], $value));
        }
        return htmlspecialchars((string) $value, ENT_QUOTES, 'UTF-8');
    }

    private function getTableHeaders() {
        return ['Field', 'Value'];
    }

    private function determineStyle($value) {
        switch ($value) {
            case 'completed':
            case 'true':
                return ['icon' => 'check-circle', 'class' => 'pass'];
            case 'failed':
            case 'false':
                return ['icon' => 'times-circle', 'class' => 'fail'];
            case 'N/A':
            case null:
                return ['icon' => 'question-circle', 'class' => 'na'];
            default:
                return ['icon' => 'info-circle', 'class' => 'info'];
        }
    }

    private function formatRowsForTable($items) {
        $rows = [];
        foreach ($items as $item) {
            foreach ($item as $key => $value) {
                $styleInfo = $this->determineStyle($value);
                $rows[] = [
                    [
                        'data' => [
                            '#type' => 'html_tag',
                            '#tag' => 'span',
                            '#value' => $key,
                            '#attributes' => ['class' => [$styleInfo['class']]]
                        ],
                    ],
                    [
                        'data' => [
                            '#markup' => $this->t('<i class="fa fa-:icon"></i> :value', [
                                ':icon' => $styleInfo['icon'], 
                                ':value' => $value
                            ]),
                            '#allowed_tags' => ['i'],
                        ],
                    ],
                ];
            }
        }
        return $rows;
    }

    private function buildDownloadLink() {
        $filePaths = $this->state->get('idealyzer.last_scan_files', []);
        if (!empty($filePaths)) {
            return [
                '#type' => 'link',
                '#title' => $this->t('Download Results'),
                '#url' => \Drupal\Core\Url::fromRoute('idealyzer.download_results'),
                '#attributes' => ['class' => ['button', 'button--primary']],
            ];
        }
        return [];
    }

    public function clearResults() {
        // Clear the stored results state
        $this->state->delete('idealyzer.api_response');
        $this->state->delete('idealyzer.last_scan_files');

        // Optionally delete files linked to old results
        $this->deleteOldFiles();
        return $this->redirect('idealyzer.admin_form');
    }

    private function deleteOldFiles() {
        $previousFiles = $this->state->get('idealyzer.previous_scan_files', []);
        foreach ($previousFiles as $file) {
            $this->fileSystem->delete($file);
        }
        $this->state->delete('idealyzer.previous_scan_files');
    }

    private function saveResults($data) {
        $timestamp = date('Ymd_His');
        $json_filename = "scan_results_{$timestamp}.json";
        $csv_filename = "scan_results_{$timestamp}.csv";
    
        // Directory check
        $directory = "public://scan_results/";
        \Drupal::service('file_system')->prepareDirectory($directory, \Drupal\Core\File\FileSystemInterface::CREATE_DIRECTORY);
    
        // Save JSON format
        $json_filepath = $directory . $json_filename;
        file_put_contents($json_filepath, json_encode($data));
    
        // Save CSV format
        $csv_filepath = $directory . $csv_filename;
        $fp = fopen($csv_filepath, 'w');
    
        if (!empty($data['items'])) {
            $headers = array_keys($data['items'][0]);
            fputcsv($fp, $headers); // Write headers
            foreach ($data['items'] as $row) {
                // Ensure each row is an array
                $row = array_map(function ($value) {
                    return is_array($value) ? json_encode($value) : $value;
                }, $row);
                fputcsv($fp, $row); // Write each row
            }
        }
    
        fclose($fp);
    
        return [
            'json' => $json_filepath,
            'csv' => $csv_filepath
        ];
    }

    public function downloadResults() {
        $filePaths = $this->state->get('idealyzer.last_scan_files', []);
        $build = [];

        if (!empty($filePaths)) {
            $links = [];
            foreach ($filePaths as $format => $path) {
                $url = \Drupal::service('file_url_generator')->generateAbsoluteString($path);
                $links[] = [
                    '#title' => $this->t('Download :format', [':format' => strtoupper($format)]),
                    '#type' => 'link',
                    '#url' => \Drupal\Core\Url::fromUri($url),
                ];
            }
            $build['downloads'] = [
                '#theme' => 'item_list',
                '#items' => $links,
                '#attributes' => ['class' => ['download-links']],
            ];
        } else {
            $build = [
                'scan' => [
                    '#type' => 'link',
                    '#title' => $this->t('Return to Site Scanner'),
                    '#url' => \Drupal\Core\Url::fromRoute('idealyzer.admin_form'),
                    '#attributes' => ['class' => ['button', 'button--primary']],
                ],
                '#type' => 'markup',
                '#markup' => $this->t('No items found. '),
            ];
        }

        return $build;
    }
}
